#!/Users/perhov/tmp/webcam/pyvenv/bin/python
# -*- coding: utf-8 -*-

import usb1



# Sidetall refererer til dokumentet "UVC 1.5 Class specification.pdf",
# se http://www.usb.org/developers/docs/devclass_docs/
# og http://www.usb.org/developers/docs/devclass_docs/USB_Video_Class_1_5.zip

# USB request types, kilde: Internett
REQUEST_SET = 0x21

# Table A-4: Video Class-Specific Descriptor Types (side 171)
CS_INTERFACE = 0x24

# Table A-5: Video Class-Specific VC Interface Descriptor Subtypes (side 172)
VC_PROCESSING_UNIT = 0x05

# Table A-1: Video Interface Class Code (side 171)
CC_VIDEO = 0x0e

# Table A-2: Video Interface Subclass Codes (side 171)
SC_VIDEOCONTROL = 0x01

# Table A-8: Video Class-Specific Request Codes
SET_CUR = 0x01

# Table A-13: Processing Unit Control Selectors (side 175) og
# Table 4-35: Power Line Frequency Control (side 113)
PU_POWER_LINE_FREQUENCY_CONTROL = 0x05
FREQ_50Hz = chr(1)
FREQ_60Hz = chr(2)



def video_control_endpoints(dev):
    """
    Leter gjennom USB-devicen sine interface descriptorer etter
    endpoint descriptorer av typen CC_VIDEO/SC_VIDEOCONTROL.
    """
    for cfg in dev:
        #print "  Configuration"
        for inum, interface in enumerate(cfg):
            #print "    Interface"
            for endpoint in interface:
                #print "      Endpoint %r" % (endpoint.getClassTuple(),)
                if endpoint.getClassTuple() == (CC_VIDEO, SC_VIDEOCONTROL):
                    yield inum, endpoint

              

def get_processing_unit_id(endpoint):
    """
    Leter gjennom (videocontrol-)endpointet sine konfigurasjonsdescriptorer
    og returnerer Unit ID til endpointets processing unit.
    """
    for descr in endpoint.getExtra():
        bDescriptorType = ord(descr[1])
        if bDescriptorType == CS_INTERFACE:
            bDescriptorSubtype = ord(descr[2]) # Side 69
            #print "        endpoint 0x%x/0x%x" % endpoint.getClassTuple()
            #print "        type 0x%x subtype 0x%x" % (bDescriptorType, bDescriptorSubtype)
            if bDescriptorSubtype == VC_PROCESSING_UNIT:
                bUnitID = ord(descr[3])
                return bUnitID
    return None



def set_50hz(dev, interface_number, bUnitID):
    bmRequestType = REQUEST_SET
    bRequest = SET_CUR
    wValue = PU_POWER_LINE_FREQUENCY_CONTROL << 8
    wIndex = bUnitID << 8 | interface_number
    data = FREQ_50Hz

    d = dev.open()
    d.controlWrite(bmRequestType, bRequest, wValue, wIndex, data)
    d.close()



def main():
    n = 0
    ctx = usb1.USBContext()
    for dev in ctx.getDeviceList():
        print "Found USB device %04x:%04x" % (dev.getVendorID(), dev.getProductID())
        for inum, iface in video_control_endpoints(dev):
            print "  This is a webcam (%s)" % dev.getProduct()
            unit_id = get_processing_unit_id(iface)
            if unit_id is None:
                print "  Webcam lacks processing unit"
            else:
                print "  Setting power line frequency to 50 Hz:"
                set_50hz(dev, inum, unit_id)
                n += 1
    print
    print "%d web cameras changed" % n


if __name__ == '__main__':
    main()
